/**
 * Tecnología del ocio
 * Bloque 1, unidad didáctica 9
 * Práctica 2
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <cwiid.h>
#include <math.h>

cwiid_mesg_callback_t cwiid_callback; //Función para callback
cwiid_err_t error; //Función para errores

struct acc_cal calibracion; //Datos de calibración

//Función para errores
void error(cwiid_wiimote_t *wiimote, const char *s, va_list ap) {
	if (wiimote != NULL) {printf("Error en el dispositivo %d: ", cwiid_get_id(wiimote));}
	vprintf(s, ap);
	printf("\n");
}

//Función de callback
void cwiid_callback(cwiid_wiimote_t *wiimote, int mesg_count, union cwiid_mesg mesg[], struct timespec *timestamp) {
	int i, j;
	double accx, accy, accz, pitch, roll;

	//Bucle que recorre todos los mensajes incluidos en la llamada al callback
	for (i = 0; i < mesg_count; i++) {
		if (mesg[i].type == CWIID_MESG_ACC) {
			//Obtención de los datos calibrados
			accx = (double)(mesg[i].acc_mesg.acc[CWIID_X] - calibracion.zero[CWIID_X])/(double)(calibracion.one[CWIID_X] - calibracion.zero[CWIID_X]);
			accy = (double)(mesg[i].acc_mesg.acc[CWIID_Y] - calibracion.zero[CWIID_Y])/(double)(calibracion.one[CWIID_Y] - calibracion.zero[CWIID_Y]);
			accz = (double)(mesg[i].acc_mesg.acc[CWIID_Z] - calibracion.zero[CWIID_Z])/(double)(calibracion.one[CWIID_Z] - calibracion.zero[CWIID_Z]);
			//Suma de 3 (máximo del ADXL330) y recorte para evitar valores negativos
			if (accx < -3.0) {accx = -3.0;} else {accx += 3.0;}
			if (accy < -3.0) {accy = -3.0;} else {accy += 3.0;}
			if (accz < -3.0) {accz = -3.0;} else {accz += 3.0;}
			//Representación de los datos
			for (j = 0; j < ceil(accx*10); j++) {printf(" ");} printf("X\n");
			for (j = 0; j < ceil(accy*10); j++) {printf(" ");} printf("Y\n");
			for (j = 0; j < ceil(accz*10); j++) {printf(" ");} printf("Z\n");
		}
		else {
			printf("Tipo de mensaje no esperado\n");
		}
	}
}


int main(int argc, char *argv[]) {

	cwiid_wiimote_t* wiimote; //Gestor del wiimote
	bdaddr_t bdaddr; //Dirección Bluetooth del wiimote
	int salir = 0; //Variable para interrumpir la ejecución
	char tmp[256]; //Temporal para mostrar la dirección Bluetooth

	cwiid_set_err(error); //Asigno la función de error

	printf("Poner el wiimote en modo de descubrimiento (pulsar 1+2) y pulsar enter...\n");
	getchar();
	printf("Buscando...\n");
	bdaddr = *BDADDR_ANY; //Se aceptará la conexión de cualquier wiimote
	if (!(wiimote = cwiid_connect(&bdaddr, CWIID_FLAG_MESG_IFC))) { //Conexión y habilitación de envío de mensajes
		printf("No se pudo detectar wiimote\n");
	}
	else {
		//Conexión establecida
		ba2str(&bdaddr, tmp); //Traducción a cadena de caracteres de la dirección Bluetooth
		printf("Dirección Bluetooth del wiimote %i: %s\n", cwiid_get_id(wiimote), tmp);
		if (cwiid_set_mesg_callback(wiimote, cwiid_callback)) { //Establezco el callback
			printf("No se pudo establecer el callback\n");
		}
		else {
			if (cwiid_get_acc_cal(wiimote, CWIID_EXT_NONE, &calibracion)) {
				printf("No se pudieron obtener los datos de calibración\n");
			}
			else {
				if (cwiid_set_rpt_mode(wiimote, CWIID_RPT_ACC)) {
					printf("Error al establecer el modo de informe.\n");
				}
				else {
					printf("Escribe x y pulsa enter para salir\n");
					while (!salir) {
						//Bucle principal, sólo espera
						if (getchar() == 'x') {salir = 1;}
					}
				}
			}
		}
		if (cwiid_disconnect(wiimote)) { //Desconexión del wiimote
			printf("Error al desconectar el wiimote\n");
		}
	}
	return 0;
}
