/**
 * Tecnología del ocio
 * Bloque 1, unidad didáctica 9
 * Práctica 3
 */

#include <math.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <cwiid.h>

cwiid_err_t error; //Función para errores

//Función para errores
void error(cwiid_wiimote_t *wiimote, const char *s, va_list ap) {
	if (wiimote != NULL) {printf("Error en el dispositivo %d: ", cwiid_get_id(wiimote));}
	vprintf(s, ap);
	printf("\n");
}

int main(int argc, char *argv[]) {

	cwiid_wiimote_t* wiimote; //Gestor del Wiimote
	bdaddr_t bdaddr; //Dirección Bluetooth del Wiimote
	struct cwiid_state estado; //Estado del Wiimote
	char tmp[256]; //Temporal para mostrar la dirección Bluetooth
	char c; //Caracter escrito
	int angulo; //Ángulo sorteado
	int salir = 0; //Finaliza el bucle principal
	struct acc_cal calibracion; //Datos de calibración
	double accx, accy, accz; //Aceleraciones
	double cabeceo; //Cabeceo obtenido

	cwiid_set_err(error); //Asigno la función de error

	printf("Poner el Wiimote en modo de descubrimiento (pulsar 1+2) y pulsar enter...\n");
	getchar();
	printf("Buscando...\n");
	bdaddr = *BDADDR_ANY; //Se aceptará la conexión de cualquier Wiimote
	if (!(wiimote = cwiid_connect(&bdaddr, 0))) { //Conexión y habilitación de envío de mensajes
		printf("No se pudo detectar wiimote\n");
	}
	else {
		//Conexión establecida
		ba2str(&bdaddr, tmp); //Traducción a cadena de caracteres de la dirección Bluetooth
		printf("Dirección Bluetooth del Wiimote %i: %s\n", cwiid_get_id(wiimote), tmp);
		if (cwiid_get_acc_cal(wiimote, CWIID_EXT_NONE, &calibracion)) {
			printf("No se pudieron obtener los datos de calibración\n");
		}
		else {
			if (cwiid_set_rpt_mode(wiimote, CWIID_RPT_ACC)) {
				printf("Error al establecer el modo de informe.\n");
				salir = 1; //Evita que entre en el bucle
			}
			srand(time(NULL)); //Inicializar la semilla aleatoria
			while (!salir) {
				angulo = ((int)rand() % 10)*10;
				printf("Situa el Wiimote con %d grados de cabeceo (pitch) y pulsa enter cuando creas que lo has conseguido. No se tendrá en cuenta el signo.\n", angulo);
				printf("Para salir pulsa x y enter.\n");
				c = getchar();
				if (c == 'x') {
					salir = 1;
				}
				else {
					if (cwiid_get_state(wiimote, &estado) != 0) {
						printf("Error al obtener el estado del Wiimote\n");
					}
					else {
						//Obtención de los datos calibrados
						accx = (double)(estado.acc[CWIID_X] - calibracion.zero[CWIID_X])/(double)(calibracion.one[CWIID_X] - calibracion.zero[CWIID_X]);
						accy = (double)(estado.acc[CWIID_Y] - calibracion.zero[CWIID_Y])/(double)(calibracion.one[CWIID_Y] - calibracion.zero[CWIID_Y]);
						accz = (double)(estado.acc[CWIID_Z] - calibracion.zero[CWIID_Z])/(double)(calibracion.one[CWIID_Z] - calibracion.zero[CWIID_Z]);
						cabeceo = atan2(accy, sqrt(pow(accx,2) + pow(accz,2)))*180/M_PI;
						if (abs(fabs(cabeceo)-(double)angulo) < 10) {
							printf("¡Conseguido! Cabeceo = %f.\n", cabeceo);
						}
						else {
							printf("Fallo. Cabeceo = %f.\n", cabeceo);
						}
					}
				}
			}
		}
		if (cwiid_disconnect(wiimote)) { //Desconexión del Wiimote
			printf("Error al desconectar el Wiimote\n");
		}
	}
	return 0;
}
